@@ -122,6 +122,8 @@ class Entity(object):
             name_parts = name.split('.')
             o = self
             for n in name_parts:
+                if o is None:
+                    break
                 o = getattr(o, n)
             val = self.simplify_object(o, n)
             if name == as_name:
@@ -232,11 +234,11 @@ class Entity(object):
             self.__public_names
         except AttributeError:
             names = []
-            class_attrs = self.__class__.__dict__
+            class_attrs = dir(self.__class__)
             for name in class_attrs:
                 if name.startswith('_'):
                     continue
-                attr = class_attrs[name]
+                attr = getattr(self.__class__, name)
                 if isinstance(attr, property):
                     names.append(name)
                 else:
